// MQNZRTOQ.CPP program

//*********************************************************************************************
//*                                                                                           *
//*                            MQNZRTOQ.CPP program                                           *
//*                            ====================                                           *
//*                                                                                           *
//*   This program will Zero-Length records to a Queue (tagged or not).                       *
//*                                                                                           *
//*   The Zero-Length record will act as an End of Queue Indication (or also a Trigger).      *
//*                                                                                           *
//*                                                                                           *
//*   The parameters are  /Maaaaaa    -> the name of the source Queue Manager                 *
//*                                                  default -> blanks (default QM)           *
//*                                                                                           *
//*                       /Qaaaaaa    -> the name of the source Queue                         *
//*                                                  default -> blanks                        *
//*                                                                                           *
//*                       /Taaaaaa    -> the MSGID of the zero-Length record                  *
//*                                                  /T+     -> all EBCDIC spaces             *
//*                                                  /T-     -> all ASCII  spaces             *
//*                                                  default -> all ASCII  nulls              *
//*                                                                                           *
//*                       /Pd         -> Control the persistance of the messages. /P+ forces  *
//*                       /P+            Persistant Messages, /P- forces non-Persistance, and *
//*                       /P-            /Pd uses the default Persistance from the Queue      *
//*                                                                                           *
//*                       /Cn         -> Number of Zero Length records to write to the Queue  *
//*                                                     (default 1)                           *
//*                                                                                           *
//*                       /V+         -> Write Diagnotic Messages (default)                   *
//*                       /V-         -> Don't display any messages                           *
//*                                                                                           *
//*                                                                                           *
//*   Example : MQNZRTOQ /MRAHQM1 /QRAH1 /T- /Pd             will add a Zero-Length record    *
//*                                                          into queue RAH1 owned by         *
//*                                                          Queue Manager RAHQM1. The        *
//*                                                          MsgID of the message is all      *
//*                                                          X'20's.                          *
//*                                                                                           *
//*   Standard Messages are written to show information if /V+ is specified.                  *
//*                                                                                           *
//*                                                                                           *
//*                                                                                           *
//*   Return Codes :    0    -> Operation succeeded                                           *
//*                     1    -> No Zero Length records were requested (/C0)                   *
//*                     >0   -> Reason Code for a failure                                     *
//*                                                                                           *
//*                                                                                           *
//*   Copyright : This program is Copyrighted by IBM UK Labs Limited, 1994.                   *
//*                                                                                           *
//*                                                                                           *
//*   Support   : This program is not supported in any way by IBM. It is distributed          *
//*               only to show techniques for Message Queueing. It may be freely              *
//*               modified and adapted by your installation.                                  *
//*                                                                                           *
//*   Author    : Robert Harris,                                                              *
//*               Object Technology Products,                                                 *
//*               IBM Hursley Park,                                                           *
//*               England.                                                                    *
//*                                                                                           *
//*********************************************************************************************

//
// System Includes
//

#include <stdlib.h>
#include <stdio.h>
#include <iostream.h>
#include <time.h>

//
// MQ Includes
//

#include <cmqc.h>

//
// Windows Includes
//

#include <windows.h>

//
// Definitions of datatypes
//

#define FULLWORD     long
#define HALFWORD     int
#define MAXSFULLWORD (68719476735)
#define MAXUFULLWORD (4294967295)
#define MAXSHALFWORD (4095)
#define MAXUHALFWORD (65535)
#define MAXFILENAME  (255)

//
// The parms class contains the unravelling of the
//     supplied parameters
//

class parms
{
   private:

       FULLWORD    verbose                          ; // V flag 0=- 1=+
       FULLWORD    persist                          ; // P flag 0=default 1=Pers 2=NotP
       FULLWORD    zeronum                          ; // C n number of records
       char        tag??(25??)                      ; // T tag required
       char        dqm ??(MQ_Q_MGR_NAME_LENGTH+1??) ; // QM    name - destination
       char        dq  ??(MQ_Q_NAME_LENGTH+1    ??) ; // Q     name - destination

       FULLWORD parms::getnumb( char * const ptr ) ;    //Private func to get nums out of parms

   public:
                 parms       () { verbose        =  1        ; //Constructor
                                  persist        =  0        ; //  defaults
                                  zeronum        =  1        ; //
                                  memset(tag,0,sizeof(tag))  ; //
                                  memset(dqm,0,sizeof(dqm))  ; //
                                  memset(dq ,0,sizeof(dq ))  ; //
                                }

       FULLWORD  isverbose   () { if ( verbose == 1 ) return(TRUE)  ;   //Access func to the pa
                                       else           return(FALSE) ; } ;

       FULLWORD  getpersist  () { return (persist) ; } ;

       FULLWORD  getnumz     () { return (zeronum) ; } ;

       char    * gettag       (char * const dest)
                              {
                                memcpy(dest,tag,sizeof(tag)) ;
                                return(dest)                 ;
                              }

       char    * getdqmname   (char * const dest)
                              {
                                strcpy(dest,dqm) ;
                                return(dest)     ;
                              }

       char    * getdqname    (char * const dest)
                              {
                                strcpy(dest,dq)  ;
                                return(dest)     ;
                              }


       void     parmset ( int     const argc,                       //Function to set
                          char ** const argv,                       // up parms in the
                          char ** const envp  ) ;                   // class

} ;

//
// The parmset function is the part of the parms class
//     that builds the parameters
//
// Valid parms are :
//
//            /Maaaaa     -> The Queue Manager Name
//            /Qaaaaa     -> The Queue Name
//            /Taaaaa     -> The Required Tag
//            /P d + -    -> The Required Message Persistance  (Q,Y,N)
//            /Cn         -> The Number of records (default 1)
//            /V + -      -> Message Displaying (Y,N)
//
void parms::parmset ( int     const argc,
                      char ** const argv,
                      char ** const envp  )
 {
    int     i = 0   ;
    char  * p = 0   ;
    char    c = ' ' ;
    char    s = ' ' ;
    char    v = ' ' ;

    for ( i=1 ; i < argc ; i++ )     // loop through all the parms
        {
          p = argv??(i??) ;          // start of current parm
          c = *p          ;          // should be the selector, so ignore it
          s = *(++p)      ;          // the parm key
          p++             ;          // start of parm data
          v = *p          ;          // first of the parm data

          switch ( s )
           {
             case    'M'  :      // Queue Manager - destination
             case    'm'  :

                             strncpy(this->dqm,p,MQ_Q_MGR_NAME_LENGTH) ;
                             break                                     ;

             case    'Q'  :      // Queue - destination
             case    'q'  :

                             strncpy(this->dq,p,MQ_Q_NAME_LENGTH) ;
                             break                                ;


             case    'P'  :      // Persistance required
             case    'p'  :

                             switch ( v )
                                {
                                  case 'd'  :     //Take from Q
                                  case 'D'  :
                                              persist = 0 ;
                                              break       ;

                                  case '+'  :     //Force Persistance
                                              persist = 1 ;
                                              break       ;

                                  case '-'  :     //Force NotPersistance
                                              persist = 2 ;
                                              break       ;

                                  default   : break       ;
                                }
                             break ;

             case    'T'  :      // Tag
             case    't'  :

                             switch ( v )
                                {
                                  case '+'  :     // EBCDIC spaces
                                              memset(tag,0x40,sizeof(tag))   ;
                                              break                          ;

                                  case '-'  :     // ASCII  spaces
                                              memset(tag,0x20,sizeof(tag))   ;
                                              break                          ;

                                  default   :     // Tag supplied
                                              memset(tag,0x00,sizeof(tag))   ;
                                              strncpy(tag,p,(sizeof(tag)-1)) ;
                                              break                          ;
                                }
                             break ;

             case    'C'  :      // Number of Zero Length messages
             case    'c'  :

                             zeronum = getnumb(p) ;
                             break                ;


             case    'v'  :      // Message usage
             case    'V'  :

                             switch ( v )
                                {

                                  case '+'  :     //Yes
                                              verbose = 1 ;
                                              break       ;

                                  case '-'  :     //No
                                              verbose = 0 ;
                                              break       ;

                                  default   : break       ;
                                }
                             break ;


             default :
                        break  ;       // unknown parm, so ignore it
           } ;

        } ;

    return ;

 }

//
// The getnumb private function within the parms class
//     merely extracts the numeric part of a /Pnnnn
//
FULLWORD parms::getnumb( char * const ptr )
 {
    FULLWORD   numb = 0   ;
    char     * p    = 0   ;
    char       c    = ' ' ;

    for ( p = ptr ; *p != '\0' ; p++ )
        {
          c = *p ;
          if ( c == ' ' ) break ;
          if ( ( c >= '0' ) && ( c <= '9' ) )
                numb = ( numb * 10 ) + ( c - '0' ) ;
        }
    return ( numb ) ;
  }



//
// RAHMQO Class Library
//

class rahmqo
 {
   public :

    char        qm  ??(MQ_Q_MGR_NAME_LENGTH+1??) ; //QM name
    char        q   ??(MQ_Q_NAME_LENGTH+1    ??) ; //Q  name
    MQHCONN     qmh                              ; //QM handle
    MQHOBJ      qh                               ; //Q  handle
    MQLONG      rc                               ; //Return Code
    MQLONG      ac                               ; //Reason Code
    MQLONG      lastrc                           ; //Prev Return Code
    MQLONG      lastac                           ; //Prev Reason Code


    rahmqo ( void )               ;  //Contructor for new  use of QM
    rahmqo ( MQHCONN const uqmh)  ;  //Contructor for same use of QM

    MQLONG      conn ( void )               ;  //Connect    to QM
    MQLONG      conn ( char * const uqm )   ;  //  with QM name supplied

    MQLONG      disc ( void )               ;  //Disconnect from QM

    void        cleanup ( void )            ;  //Close and Disconnect

    MQOD        odopen                      ;  //OD for MQOPEN usage
    MQLONG      open (MQLONG const uopts )  ;  //MQOPEN call
    MQLONG      open (char * const uq    ,     //  with Q name supplied
                      MQLONG const uopts )  ;  //
    MQLONG      open (MQOD * const uod   ,     //  with OD and options
                      MQLONG const uopts )  ;  //

    MQLONG      close (MQLONG const uopts ) ;  //MQCLOSE call

    MQLONG      commit   ( void ) ; //Syncpoint UOW
    MQLONG      rollback ( void ) ; //Backout   UOW

    MQLONG      inq ( MQLONG   const  uselnum ,             //MQINQ call
                      MQLONG          usels   ,
                      MQLONG   const  uinum   ,
                      MQLONG *        uiattr  ,
                      MQLONG   const  ucharl  ,
                      char   *        ucattr  ) ;

    MQLONG      set ( MQLONG   const uselnum  ,             //MQSET call
                      MQLONG         usels    ,
                      MQLONG   const uinum    ,
                      MQLONG *       uiattr   ,
                      MQLONG   const ucharl   ,
                      char   *       ucattr   ) ;

    MQMD        pmd                             ;           //PUT Message Dwescriptor
    MQLONG      put ( MQPMO  * const  uopts   ,             //MQPUT call
                      MQLONG   const  bufferl ,
                      char   * const  buffer  ) ;

    MQLONG      put ( MQMD   *        upmd    ,
                      MQPMO  * const  uopts   ,
                      MQLONG   const  bufferl ,
                      char   * const  buffer  ) ;

                                                            //PUT Zero-Length record
    MQLONG      putz ( MQPMO  * const  uopts     ) ;        //No Tag, Use Instance MD

    MQLONG      putz ( MQMD   *        upmd      ,          //No Tag, use supplied MD
                       MQPMO  * const  uopts     ) ;

    MQLONG      putz ( MQPMO    * const  uopts   ,          //Tag, Use Instance MD
                       MQBYTE24 * const  utag    ) ;

    MQLONG      putz ( MQMD     *        upmd    ,          //Tag, use supplied MD
                       MQPMO    * const  uopts   ,
                       MQBYTE24 * const  utag    ) ;

    MQMD        bmd                                  ;      //Browse Message Descriptor
    MQGMO       bmo                                  ;      //Browse MQGET options
    MQLONG      browse ( MQLONG   const  bufferl   ,
                         char   * const  buffer    ,
                         MQLONG *        udatalen  ) ;

    MQLONG      browse ( MQMD   *        ubmd      ,
                         MQLONG   const  bufferl   ,
                         char   * const  buffer    ,
                         MQLONG *        udatalen  ) ;


    MQMD        gmd                               ;         //GET Message Descriptor
    MQLONG      get ( MQGMO  * const  uopts     ,           //MQGET call
                      MQLONG   const  bufferl   ,
                      char   * const  buffer    ,
                      MQLONG *        udatalen  ) ;

    MQLONG      get ( MQMD   *        ugmd      ,
                      MQGMO  * const  uopts     ,
                      MQLONG   const  bufferl   ,
                      char   * const  buffer    ,
                      MQLONG *        udatalen  ) ;


    double      gettimeconn     () { return (time_conn     ) ; }
    double      gettimedisc     () { return (time_disc     ) ; }
    double      gettimeopen     () { return (time_open     ) ; }
    double      gettimeclose    () { return (time_close    ) ; }
    double      gettimeinq      () { return (time_inq      ) ; }
    double      gettimeset      () { return (time_set      ) ; }
    double      gettimeput      () { return (time_put      ) ; }
    double      gettimeget      () { return (time_get      ) ; }
    double      gettimecommit   () { return (time_commit   ) ; }
    double      gettimerollback () { return (time_rollback ) ; }
    double      getcputconn     () { return (cput_conn     / CLOCKS_PER_SEC) ; }
    double      getcputdisc     () { return (cput_disc     / CLOCKS_PER_SEC) ; }
    double      getcputopen     () { return (cput_open     / CLOCKS_PER_SEC) ; }
    double      getcputclose    () { return (cput_close    / CLOCKS_PER_SEC) ; }
    double      getcputinq      () { return (cput_inq      / CLOCKS_PER_SEC) ; }
    double      getcputset      () { return (cput_set      / CLOCKS_PER_SEC) ; }
    double      getcputput      () { return (cput_put      / CLOCKS_PER_SEC) ; }
    double      getcputget      () { return (cput_get      / CLOCKS_PER_SEC) ; }
    double      getcputcommit   () { return (cput_commit   / CLOCKS_PER_SEC) ; }
    double      getcputrollback () { return (cput_rollback / CLOCKS_PER_SEC) ; }

 private :

    double      time_conn      ;
    double      time_disc      ;
    double      time_open      ;
    double      time_close     ;
    double      time_inq       ;
    double      time_set       ;
    double      time_put       ;
    double      time_get       ;
    double      time_commit    ;
    double      time_rollback  ;


    double      cput_conn      ;
    double      cput_disc      ;
    double      cput_open      ;
    double      cput_close     ;
    double      cput_inq       ;
    double      cput_set       ;
    double      cput_put       ;
    double      cput_get       ;
    double      cput_commit    ;
    double      cput_rollback  ;
 } ;

//
// rahmqo member function definitions
//

//
// Constructor functions
//
//     without a parm, then this is a new use of a QM
//     with          , then this is an additional access on a
//                     previously connected QM
//

rahmqo::rahmqo ( void )
{
  memset(qm,0,sizeof(qm)) ;
  memset(q ,0,sizeof(q )) ;
  qmh    = 0              ;
  qh     = 0              ;
  rc     = 0              ;
  ac     = 0              ;
  lastrc = 0              ;
  lastac = 0              ;
  memset(&odopen,0,sizeof(odopen))                             ;
  strncpy(odopen.StrucId,MQOD_STRUC_ID,sizeof(odopen.StrucId)) ;
  odopen.Version    = MQOD_VERSION_1                           ;
  odopen.ObjectType = MQOT_Q                                   ;
  memset(&pmd,0,sizeof(pmd))                                   ;
  memset(&bmd,0,sizeof(bmd))                                   ;
  memset(&gmd,0,sizeof(gmd))                                   ;

  cput_conn      = 0 ;
  cput_disc      = 0 ;
  cput_open      = 0 ;
  cput_close     = 0 ;
  cput_inq       = 0 ;
  cput_set       = 0 ;
  cput_put       = 0 ;
  cput_get       = 0 ;
  cput_commit    = 0 ;
  cput_rollback  = 0 ;
  time_conn      = 0 ;
  time_disc      = 0 ;
  time_open      = 0 ;
  time_close     = 0 ;
  time_inq       = 0 ;
  time_set       = 0 ;
  time_put       = 0 ;
  time_get       = 0 ;
  time_commit    = 0 ;
  time_rollback  = 0 ;

  return             ;
}

rahmqo::rahmqo ( MQHCONN const uqmh )
{
  memset(qm,0,sizeof(qm)) ;
  memset(q ,0,sizeof(q )) ;
  qmh    = uqmh           ;
  qh     = 0              ;
  rc     = 0              ;
  ac     = 0              ;
  lastrc = 0              ;
  lastac = 0              ;
  memset(&odopen,0,sizeof(odopen))                             ;
  strncpy(odopen.StrucId,MQOD_STRUC_ID,sizeof(odopen.StrucId)) ;
  odopen.Version    = MQOD_VERSION_1                           ;
  odopen.ObjectType = MQOT_Q                                   ;
  memset(&pmd,0,sizeof(pmd))                                   ;
  memset(&bmd,0,sizeof(bmd))                                   ;
  memset(&gmd,0,sizeof(gmd))                                   ;

  cput_conn      = 0 ;
  cput_disc      = 0 ;
  cput_open      = 0 ;
  cput_close     = 0 ;
  cput_inq       = 0 ;
  cput_set       = 0 ;
  cput_put       = 0 ;
  cput_get       = 0 ;
  cput_commit    = 0 ;
  cput_rollback  = 0 ;
  time_conn      = 0 ;
  time_disc      = 0 ;
  time_open      = 0 ;
  time_close     = 0 ;
  time_inq       = 0 ;
  time_set       = 0 ;
  time_put       = 0 ;
  time_get       = 0 ;
  time_commit    = 0 ;
  time_rollback  = 0 ;

  return             ;
}

//
// rahmqo destructor functions
//
//   these are not provided
//
//    (Note that a disc call removes access to ALL
//     instances of RAHMQO sharing the Handle. as I
//     am not aiming at a 'proper' C++ implementation
//     I have not provided code to cope with this event).
//

//
// rahmqo functions to access a Queue Manager
//

//
// conn  will connect to a Queue Manager
//       saving the QMH
//
//       The version with no parms will use the
//           pre-set Queue Manager name
//
//       The alternative access is to quote a
//           Queue Manager Name which is the
//           one to be used
//

MQLONG  rahmqo::conn ( char * const uqm )
{
  double c1, c2, t1, t2         ;
  strncpy(qm,uqm,sizeof(qm)-1)  ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQCONN ( qm, &qmh, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_conn += 1.0000 * ( c2 - c1 )      ;
  time_conn += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}


MQLONG  rahmqo::conn ( void )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQCONN ( qm, &qmh, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_conn += 1.0000 * ( c2 - c1 )      ;
  time_conn += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}


//
// disc  will disconnect from a Queue Manager
//       deleting the QMH
//


MQLONG  rahmqo::disc ( void )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQDISC ( &qmh, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_disc += 1.0000 * ( c2 - c1 )      ;
  time_disc += 1.0000 * difftime(t2,t1)  ;

  qmh = 0     ;

  return (ac) ;
}

//
// cleanup will Close the Q (if known) and Disconnect from
//         the QM (if known). This will effect other instances
//         which were created with a QM handle
//

void    rahmqo::cleanup ( void )
{
  double c1, c2, t1, t2         ;

  if ( qh  != 0 ) close(MQCO_NONE) ;
  if ( qmh != 0 ) disc ()          ;
  return                           ;
}

//
// commit will Syncpoint ALL accesses to the
//       QM on the given QMH (this will effect
//       other instances of the Class if the
//       QMH is being shared) in the current UOW
//

MQLONG  rahmqo::commit ( void )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQCMIT ( qmh, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_commit += 1.0000 * ( c2 - c1 )      ;
  time_commit += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}

//
// rollback will backout ALL accesses to the
//       QM on the given QMH (this will effect
//       other instances of the Class if the
//       QMH is being shared) in the current UOW
//

MQLONG  rahmqo::rollback ( void )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQBACK ( qmh, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_rollback += 1.0000 * ( c2 - c1 )      ;
  time_rollback += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}

//
// open  will provide access to a Queue
//
//       The OPEN options are always supplied
//           by the caller.
//
//       If the version without a Queue Name is
//          used, then the Name from the Instance
//          is taken.
//

MQLONG  rahmqo::open ( MQLONG const uopts )
{
  double c1, c2, t1, t2         ;

  memset(&odopen,0,sizeof(odopen))                             ;
  strncpy(odopen.StrucId,MQOD_STRUC_ID,sizeof(odopen.StrucId)) ;
  odopen.Version    = MQOD_VERSION_1                           ;
  odopen.ObjectType = MQOT_Q                                   ;
  strcpy(odopen.ObjectName,q)                                  ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQOPEN ( qmh, &odopen, uopts, &qh, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_open += 1.0000 * ( c2 - c1 )      ;
  time_open += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}

MQLONG  rahmqo::open ( char * const uq    ,
                       MQLONG const uopts )
{
  double c1, c2, t1, t2         ;

  strncpy(q,uq,sizeof(q)-1)   ;

  memset(&odopen,0,sizeof(odopen))                             ;
  strncpy(odopen.StrucId,MQOD_STRUC_ID,sizeof(odopen.StrucId)) ;
  odopen.Version    = MQOD_VERSION_1                           ;
  odopen.ObjectType = MQOT_Q                                   ;
  strcpy(odopen.ObjectName,q)                                  ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQOPEN ( qmh, &odopen, uopts, &qh, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_open += 1.0000 * ( c2 - c1 )      ;
  time_open += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}

MQLONG  rahmqo::open ( MQOD * const uod   ,
                       MQLONG const uopts )
{
  double c1, c2, t1, t2         ;

  memcpy(&odopen,uod,sizeof(odopen)) ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQOPEN ( qmh, &odopen, uopts, &qh, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_open += 1.0000 * ( c2 - c1 )      ;
  time_open += 1.0000 * difftime(t2,t1)  ;

  memcpy(uod,&odopen,sizeof(odopen))     ;
  return (ac) ;
}

//
// close will stop access to a Queue
//
//       The CLOSE options are always supplied
//           by the caller.
//

MQLONG  rahmqo::close ( MQLONG const uopts )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQCLOSE ( qmh, &qh, uopts, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_close += 1.0000 * ( c2 - c1 )      ;
  time_close += 1.0000 * difftime(t2,t1)  ;

  qh = 0      ;

  return (ac) ;
}

//
// inq   will inquire upon the Attributes of
//       a Queue etc.
//
//       Note that ALL the Selectors stuff
//       has to be supplied by the caller
//

MQLONG  rahmqo::inq ( MQLONG   const  uselnum ,
                      MQLONG          usels   ,
                      MQLONG   const  uinum   ,
                      MQLONG *        uiattr  ,
                      MQLONG   const  ucharl  ,
                      char   *        ucattr  )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQINQ ( qmh, qh, uselnum, &usels ,
                   uinum  , uiattr ,
                   ucharl , ucattr ,
                                     &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_inq += 1.0000 * ( c2 - c1 )      ;
  time_inq += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}
//
// set   will set the Attributes of a Queue etc.
//
//       Note that ALL the Selectors stuff
//       has to be supplied by the caller
//

MQLONG  rahmqo::set ( MQLONG   const   uselnum ,
                      MQLONG           usels   ,
                      MQLONG   const   uinum   ,
                      MQLONG *         uiattr  ,
                      MQLONG   const   ucharl  ,
                      char   *         ucattr  )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQSET ( qmh, qh, uselnum , &usels  ,
                   uinum   , uiattr  ,
                   ucharl  , ucattr  ,
                                       &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_set += 1.0000 * ( c2 - c1 )      ;
  time_set += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}


//
// put       will write a Message into a Queue
//
//           The Message descriptor may either
//           be supplied or the instance version
//           used.
//

MQLONG  rahmqo::put ( MQPMO  * const  uopts   ,
                      MQLONG   const  bufferl ,
                      char   * const  buffer  )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQPUT ( qmh, qh, &pmd, uopts, bufferl, buffer, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_put += 1.0000 * ( c2 - c1 )      ;
  time_put += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}


MQLONG  rahmqo::put ( MQMD   *        upmd    ,
                      MQPMO  * const  uopts   ,
                      MQLONG   const  bufferl ,
                      char   * const  buffer  )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQPUT ( qmh, qh, upmd, uopts, bufferl, buffer, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_put += 1.0000 * ( c2 - c1 )      ;
  time_put += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}

//
// putz      will write a Zero-Length Message into a Queue
//
//           If a Tag is supplied, then this is used as the Message Descriptor
//              this must be an area of MQBYTE24 (24 bytes long)
//
//           The Message descriptor may either
//           be supplied or the instance version
//           used.
//

MQLONG  rahmqo::putz ( MQPMO  * const  uopts )               //No Tag, Use Instance MD
{
  double c1, c2, t1, t2         ;

  memset(&this->pmd.MsgId,0,sizeof(this->pmd.MsgId)) ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQPUT ( qmh, qh, &pmd, uopts, 0, 0, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_put += 1.0000 * ( c2 - c1 )      ;
  time_put += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}


MQLONG  rahmqo::putz ( MQMD   *        upmd    ,              //No Tag, use supplied MD
                       MQPMO  * const  uopts   )
{
  double c1, c2, t1, t2         ;

  memset(upmd->MsgId,0,sizeof(upmd->MsgId)) ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQPUT ( qmh, qh, upmd, uopts, 0, 0, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_put += 1.0000 * ( c2 - c1 )      ;
  time_put += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}

MQLONG  rahmqo::putz ( MQPMO    * const  uopts   ,            //Tag, Use Instance MD
                       MQBYTE24 * const  utag    )
{
  double c1, c2, t1, t2         ;

  memcpy(&this->pmd.MsgId,utag,sizeof(this->pmd.MsgId)) ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQPUT ( qmh, qh, &pmd, uopts, 0, 0, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_put += 1.0000 * ( c2 - c1 )      ;
  time_put += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}


MQLONG  rahmqo::putz ( MQMD     *        upmd    ,            //Tag, use supplied MD
                       MQPMO    * const  uopts   ,
                       MQBYTE24 * const  utag    )
{
  double c1, c2, t1, t2         ;

  memcpy(upmd->MsgId,utag,sizeof(upmd->MsgId)) ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQPUT ( qmh, qh, upmd, uopts, 0, 0, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_put += 1.0000 * ( c2 - c1 )      ;
  time_put += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}

//
//    Browse    will non-destructively read a Queue
//
//              Versions are supplied that use an
//              instance Message dewscriptor, or
//              one supplied by the caller
//

MQLONG  rahmqo::browse ( MQLONG   const  bufferl   ,
                         char   * const  buffer    ,
                         MQLONG *        udatalen  )
{
  double c1, c2, t1, t2         ;

  memset(&bmo,0,sizeof(bmo))                                 ;
  memcpy(&bmo.StrucId,MQGMO_STRUC_ID,sizeof(bmo.StrucId))    ;
  bmo.Version        = MQGMO_VERSION_1                       ;
  bmo.Options        = MQGMO_NO_WAIT+MQGMO_BROWSE_NEXT+MQGMO_ACCEPT_TRUNCATED_MSG+
                       MQGMO_FAIL_IF_QUIESCING               ;
  bmo.WaitInterval   = MQWI_UNLIMITED                        ;

  memset(&bmd,0,sizeof(bmd))                                 ;
  memcpy(&bmd.StrucId,MQMD_STRUC_ID,sizeof(bmd.StrucId))     ;
  bmd.Version        = MQMD_VERSION_1                        ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQGET ( qmh, qh, &bmd, &bmo, bufferl, buffer, udatalen, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_get += 1.0000 * ( c2 - c1 )      ;
  time_get += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}


MQLONG  rahmqo::browse ( MQMD   *        ubmd      ,
                         MQLONG   const  bufferl   ,
                         char   * const  buffer    ,
                         MQLONG *        udatalen  )
{
  double c1, c2, t1, t2         ;

  memset(&bmo,0,sizeof(bmo))                                 ;
  memcpy(&bmo.StrucId,MQGMO_STRUC_ID,sizeof(bmo.StrucId))    ;
  bmo.Version        = MQGMO_VERSION_1                       ;
  bmo.Options        = MQGMO_NO_WAIT+MQGMO_BROWSE_NEXT+MQGMO_ACCEPT_TRUNCATED_MSG+
                       MQGMO_CONVERT+MQGMO_FAIL_IF_QUIESCING ;
  bmo.WaitInterval   = MQWI_UNLIMITED                        ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQGET ( qmh, qh, ubmd, &bmo, bufferl, buffer, udatalen, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_get += 1.0000 * ( c2 - c1 )      ;
  time_get += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}


MQLONG  rahmqo::get ( MQGMO  * const  uopts     ,           //MQGET call
                      MQLONG   const  bufferl   ,
                      char   * const  buffer    ,
                      MQLONG *        udatalen  )
{
  double c1, c2, t1, t2         ;

  memset(&gmd,0,sizeof(gmd))                                 ;
  memcpy(&gmd.StrucId,MQMD_STRUC_ID,sizeof(gmd.StrucId))     ;
  bmo.Version        = MQMD_VERSION_1                        ;

  memset(&gmd,0,sizeof(gmd))                                 ;
  memcpy(&gmd.StrucId,MQMD_STRUC_ID,sizeof(gmd.StrucId))     ;
  gmd.Version        = MQMD_VERSION_1                        ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQGET ( qmh, qh, &gmd, uopts, bufferl, buffer, udatalen, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_get += 1.0000 * ( c2 - c1 )      ;
  time_get += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}


MQLONG  rahmqo::get ( MQMD   *        ugmd      ,
                      MQGMO  * const  uopts     ,
                      MQLONG   const  bufferl   ,
                      char   * const  buffer    ,
                      MQLONG *        udatalen  )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQGET ( qmh, qh, ugmd, uopts, bufferl, buffer, udatalen, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_get += 1.0000 * ( c2 - c1 )      ;
  time_get += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}

//
// End of RAHMQO Class definition and functions
//


//
// Main procedure - Write a Zero Length REcord to a Q
//
int  main ( int     const argc,
            char ** const argv,
            char ** const envp  )
{
 parms       myparms         ;
 rahmqo      dmq             ;

 char        dqm     ??(MQ_Q_MGR_NAME_LENGTH+1??) ; //QM name - destination
 char        dq      ??(MQ_Q_NAME_LENGTH+1    ??) ; //Q  name
 char        reqtag  ??(25??)                     ; //required Tag

 FULLWORD    numrecs  = 1    ;
 FULLWORD    persist  = 0    ;
 MQLONG      popts    = 0    ;
 MQGMO       gmo             ;
 MQPMO       pmo             ;

 double      totcput  = 0    ;
 double      tottime  = 0    ;
 double      totc1    = 0    ;
 double      totc2    = 0    ;
 double      tote1    = 0    ;
 double      tote2    = 0    ;

 MQLONG      mqrc     = 0    ;
 MQLONG      mqac     = 0    ;
 MQLONG      ac       = 0    ;
 MQLONG      rc       = 0    ;

 totc1 = clock()    ;                      //Set Timers
 tote1 = time(NULL) ;

 myparms.parmset(argc, argv, envp)      ;  //set the parms

 myparms.getdqmname(dqm)          ;    //Get the data
 myparms.getdqname (dq )          ;    // from the
 myparms.gettag    (reqtag)       ;    // parms
 persist = myparms.getpersist()   ;
 numrecs = myparms.getnumz()      ;

 if ( myparms.isverbose() )          //Display program parms
    {
      cout << "Program parms are: " ;
      for ( int i=0 ; i<argc ; cout << argv??(i++??) << " " ) ;
      cout << endl ;
    }

 if ( argc == 1 )                    //Show parms ONLY if none given
    {
      cout << "\nMQNZRTOQ will write Zero Length records to a Queue    \n\n"
           << "Parms for MQNZRTOQ are /M<queue manager>  /Q<queue>     \n"
           << "                       /T<tag>            /C<number>    \n"
           << "                       /P<persistance>    /V<verbose>   \n\n"
           << " /T+ EBCDIC spaces     /T- ASCII  spaces       /T  no Tag                \n"
           << " /Pd from Qdef         /P+ Force Persistance   /P- Force Non-Persistance \n"
           << " /V+ display           /V- no messages \n"
           << " If /C is not supplied, then /C1 is assumed (/C0 bypasses activity) \n\n"
           ;
      return (0) ;
    }


  if ( myparms.isverbose() ) switch ( persist )        //Display Message Persistance
   {
     case 0 :
              cout << "Message Persistance taken from Q definition" << endl ;
              break                                                         ;

     case 1 :
              cout << "Message forced to be Persistant" << endl ;
              break                                             ;

     case 2 :
              cout << "Message forced to be  Non-Persistant" << endl ;
              break                                                  ;

     default:
              break ;
   }

  if ( myparms.isverbose() ) switch ( *reqtag )        //Display Tag
   {
     case 0x00  :
                   cout << "Zero-Length record MsgId is untagged (ASCII nulls)"   << endl ;
                   break                                                                  ;

     case 0x20  :
                   cout << "Zero-Length record MsgID is ASCII  spaces (X\'20\'s)" << endl ;
                   break                                                                  ;

     case 0x40  :
                   cout << "Zero-Length record MsgID is EBCDIC spaces (X\'40\'s)" << endl ;
                   break                                                                  ;

     default    :
                   cout << "Zero-Length record MsgID is " << reqtag    << " "     << endl ;
                   break                                                                  ;
   }

 if ( myparms.isverbose() ) cout << numrecs << " Zero Length record(s) to be written" <<  endl ;

 if ( numrecs <= 0 )                 //Bypass if nothing to do
    {
      if ( myparms.isverbose() ) cout << "... so leaving MQNZRTOQ (RC=1)" << endl ;
      rc = 1 ;
    }

 if ( rc == 0 )                      // Connect to QM
    {
      if ( myparms.isverbose() ) cout << "Connecting to QM " << dqm << " " ;
      ac   = dmq.conn(dqm)   ;
      mqrc = dmq.lastrc      ;
      mqac = dmq.lastrc      ;

      if ( ac != 0 ) rc = ac ;
      if ( myparms.isverbose() ) cout << "RC=" << rc << endl ;
    }


 if ( rc == 0 )                      // Open the Q for Put access
    {
      if ( myparms.isverbose() ) cout << "Opening Q " << dq << " for Output " ;
      ac   = dmq.open(dq,(MQOO_OUTPUT+MQOO_FAIL_IF_QUIESCING))                ;
      mqrc = dmq.lastrc     ;
      mqac = dmq.lastrc     ;

      if ( ac != 0 ) rc = ac ;
      if ( myparms.isverbose() ) cout << "RC=" << rc << endl ;
    }

 if ( rc == 0 ) for ( int i=0 ; i < numrecs ; i++ )  // Write Zero Length records
    {
       if ( rc == 0 )
          {
            memset(&pmo,0,sizeof(pmo))  ;               //Set Put Message Options
            memcpy(&pmo.StrucId,MQPMO_STRUC_ID,sizeof(pmo.StrucId))        ;
            pmo.Version        = MQPMO_VERSION_1                           ;
            pmo.Options        = MQPMO_SYNCPOINT                           +
                                 MQPMO_NO_CONTEXT                          +
                                 MQPMO_FAIL_IF_QUIESCING                   ;

            memset(&dmq.pmd,0,sizeof(dmq.pmd))  ;     //Set Put Message Desc
            memcpy(&dmq.pmd.StrucId,MQMD_STRUC_ID,sizeof(dmq.pmd.StrucId)) ;
            memcpy(&dmq.pmd.Format ,MQFMT_NONE   ,sizeof(dmq.pmd.Format )) ;
            memcpy(&dmq.pmd.MsgId  ,reqtag       ,sizeof(dmq.pmd.MsgId  )) ;
            dmq.pmd.Version        = MQMD_VERSION_1                        ;
            dmq.pmd.Report         = MQRO_NONE                             ;
            dmq.pmd.MsgType        = MQMT_DATAGRAM                         ;
            dmq.pmd.Expiry         = MQEI_UNLIMITED                        ;
            dmq.pmd.Feedback       = MQFB_NONE                             ;
            dmq.pmd.Encoding       = MQENC_NATIVE                          ;
            dmq.pmd.CodedCharSetId = MQCCSI_Q_MGR                          ;
            dmq.pmd.Priority       = MQPRI_PRIORITY_AS_Q_DEF               ;
            dmq.pmd.PutApplType    = MQAT_OS2                              ;
            switch ( persist )
             {
               case 0 :
                       dmq.pmd.Persistence = MQPER_PERSISTENCE_AS_Q_DEF  ;
                       break                                             ;

               case 1 :
                       dmq.pmd.Persistence = MQPER_PERSISTENT            ;
                       break                                             ;

               case 2 :
                       dmq.pmd.Persistence = MQPER_NOT_PERSISTENT        ;
                       break                                             ;

               default:
                       break                                             ;
             }

            if ( myparms.isverbose() ) cout << "Writing Zero Length record  " ;
            ac   = dmq.putz ( &pmo  )  ;
            mqrc = dmq.lastrc          ;
            mqac = dmq.lastrc          ;

            if ( ac != 0 ) rc = ac ;
            if ( myparms.isverbose() ) cout << "RC=" << rc << endl ;

          } // End of writing a single Zero length record if
    } // End of number of records to write loop

 if ( rc == 0 )                      // Commit the addition
    {
      if ( myparms.isverbose() ) cout << "Syncpointing the UoW " ;
      ac   = dmq.commit()        ;
      mqrc = dmq.lastrc          ;
      mqac = dmq.lastrc          ;

      if ( ac != 0 ) rc = ac ;
      if ( myparms.isverbose() ) cout << "RC=" << rc << endl ;

    }

 if ( rc == 0 )                      // Close the Q
    {
      if ( myparms.isverbose() ) cout << "Closing the Q " ;
      ac   = dmq.close(MQCO_NONE)  ;
      mqrc = dmq.lastrc            ;
      mqac = dmq.lastrc            ;

      if ( ac != 0 ) rc = ac ;
      if ( myparms.isverbose() ) cout << "RC=" << rc << endl ;
    }

 if ( rc == 0 )                      // Disconnect from QM
    {
      if ( myparms.isverbose() ) cout << "Disconnecting from QM " ;
      ac   = dmq.disc()            ;
      mqrc = dmq.lastrc            ;
      mqac = dmq.lastrc            ;

      if ( ac != 0 ) rc = ac ;
      if ( myparms.isverbose() ) cout << "RC=" << rc << endl ;
    }


 if ( ( rc == 0 ) && ( myparms.isverbose() ) )  //Issue Good completion message
    {

      totc2 = clock()    ;
      tote2 = time(NULL) ;
      totcput = 1.0000 * ( totc2 - totc1  ) / CLOCKS_PER_SEC   ;
      tottime = 1.0000 * difftime(tote2,tote1)                 ;

      cout << numrecs << " Zero Length record(s) added to Q " << dq << "\n" ;
      cout << "CPU time (ms) and Elapsed Time (s) per MQ verb follow:\n"                   ;
      cout << " >Connect    " << dmq.getcputconn     () << " " <<  dmq.gettimeconn     () << "\n"  ;
      cout << " >Disconnect " << dmq.getcputdisc     () << " " <<  dmq.gettimedisc     () << "\n"  ;
      cout << " >Open       " << dmq.getcputopen     () << " " <<  dmq.gettimeopen     () << "\n"  ;
      cout << " >Close      " << dmq.getcputclose    () << " " <<  dmq.gettimeclose    () << "\n"  ;
      cout << " >Commit     " << dmq.getcputcommit   () << " " <<  dmq.gettimecommit   () << "\n"  ;
      cout << " >Rollback   " << dmq.getcputrollback () << " " <<  dmq.gettimerollback () << "\n"  ;
      cout << " >Inquire    " << dmq.getcputinq      () << " " <<  dmq.gettimeinq      () << "\n"  ;
      cout << " >Set        " << dmq.getcputset      () << " " <<  dmq.gettimeset      () << "\n"  ;
      cout << " >Get        " << dmq.getcputget      () << " " <<  dmq.gettimeget      () << "\n"  ;
      cout << " >Put        " << dmq.getcputput      () << " " <<  dmq.gettimeput      () << "\n"  ;
      cout << "Total MQ CPU time "     <<  (   dmq.getcputconn     ()
                                             + dmq.getcputdisc     ()
                                             + dmq.getcputopen     ()
                                             + dmq.getcputclose    ()
                                             + dmq.getcputcommit   ()
                                             + dmq.getcputrollback ()
                                             + dmq.getcputinq      ()
                                             + dmq.getcputset      ()
                                             + dmq.getcputget      ()
                                             + dmq.getcputput      ()  ) << "s "
           << "Total MQ Elapsed time " <<  (   dmq.gettimeconn     ()
                                             + dmq.gettimedisc     ()
                                             + dmq.gettimeopen     ()
                                             + dmq.gettimeclose    ()
                                             + dmq.gettimecommit   ()
                                             + dmq.gettimerollback ()
                                             + dmq.gettimeinq      ()
                                             + dmq.gettimeset      ()
                                             + dmq.gettimeget      ()
                                             + dmq.gettimeput      ()  ) << "s\n" ;
      cout << "Total CPU " << totcput << "s, Total elapsed time " << tottime << "s" << endl ;

    }

 dmq.cleanup()                        ; //Stop all accesses

 return(rc) ;                           //Return RC

}

// End of MQNZRTOQ.CPP source
